#' Run the DrData Application
#'
#' Launches the 'DrData' interactive 'Shiny' application for statistical
#' analysis and machine learning workflows.
#'
#' @return No return value, called for side effects.
#' @export
#'
#' @examples
#' if(interactive()){
#'   run_app()
#' }
run_app <- function() {
  app_dir <- system.file("app", package = "DrData")
  if (app_dir == "") {
    stop("Could not find app directory. Try re-installing 'DrData'.", call. = FALSE)
  }
  shiny::runApp(app_dir, display.mode = "normal")
}
