% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ml_functions.R
\name{ml_metrics_regression}
\alias{ml_metrics_regression}
\title{Compute regression performance metrics}
\usage{
ml_metrics_regression(y_true, y_pred)
}
\arguments{
\item{y_true}{Numeric vector of observed values.}

\item{y_pred}{Numeric vector of predicted values.}
}
\value{
One-row \code{data.frame} with columns \code{RMSE}, \code{MAE}, \code{R2}.
}
\description{
Compute regression performance metrics
}
\examples{
ml_metrics_regression(c(1,2,3,4,5), c(1.1,1.9,3.2,3.8,5.1))
}
