% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_app.R
\name{run_app}
\alias{run_app}
\title{Run the DrData Application}
\usage{
run_app()
}
\value{
No return value, called for side effects.
}
\description{
Launches the 'DrData' interactive 'Shiny' application for statistical
analysis and machine learning workflows.
}
\examples{
if(interactive()){
  run_app()
}
}
