% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ml_functions.R
\name{build_model_formula}
\alias{build_model_formula}
\title{Build a model formula with optional interaction terms}
\usage{
build_model_formula(
  target,
  features,
  use_interactions = FALSE,
  interaction_vars = NULL
)
}
\arguments{
\item{target}{Single character: response variable name.}

\item{features}{Character vector of predictor names.}

\item{use_interactions}{Logical; add two-way interactions? Default FALSE.}

\item{interaction_vars}{Character vector of variables to interact.}
}
\value{
A \code{\link[stats]{formula}} object.
}
\description{
Build a model formula with optional interaction terms
}
\examples{
build_model_formula("mpg", c("cyl", "hp", "wt"))
build_model_formula("mpg", c("cyl","hp","wt"), TRUE, c("cyl","hp"))
}
